@@ -18,7 +18,8 @@ static CMARK_INLINE void S_blankline(cmark_renderer *renderer) {
   }
 }
 
-static void S_out(cmark_renderer *renderer, const char *source, bool wrap,
+static void S_out(cmark_renderer *renderer, cmark_node *node,
+                  const char *source, bool wrap,
                   cmark_escaping escape) {
   int length = (int)strlen(source);
   unsigned char nextc;
@@ -97,7 +98,7 @@ static void S_out(cmark_renderer *renderer, const char *source, bool wrap,
       renderer->begin_content =
           renderer->begin_content && cmark_isdigit((char)c) == 1;
     } else {
-      (renderer->outc)(renderer, escape, c, nextc);
+      (renderer->outc)(renderer, node, escape, c, nextc);
       renderer->begin_line = false;
       renderer->begin_content =
           renderer->begin_content && cmark_isdigit((char)c) == 1;
@@ -143,7 +144,8 @@ void cmark_render_code_point(cmark_renderer *renderer, uint32_t c) {
 }
 
 char *cmark_render(cmark_mem *mem, cmark_node *root, int options, int width,
-                   void (*outc)(cmark_renderer *, cmark_escaping, int32_t,
+                   void (*outc)(cmark_renderer *, cmark_node *,
+                                cmark_escaping, int32_t,
                                 unsigned char),
                    int (*render_node)(cmark_renderer *renderer,
                                       cmark_node *node,
